#! /usr/bin/python

import unittest
import os.path
from pdStarter import runPd

class JsonEncodeTests(unittest.TestCase):
    basePath = 'json-encode'
    def test_add_string(self):
        out = runPd(os.path.join(self.basePath, 'add-string.pd'))
        self.assertEquals(out, '''list { "key": "value" };
''')

    def test_add_float(self):
        out = runPd(os.path.join(self.basePath, 'add-float.pd'))
        self.assertTrue(out.startswith('''list { "key": 1.100000'''))

    def test_replace(self):
        out = runPd(os.path.join(self.basePath, 'replace.pd'))
        self.assertEquals(out, '''list { "key": "value2" };
''')
    
    def test_add_array(self):
        out = runPd(os.path.join(self.basePath, 'add-array.pd'))
        self.assertEquals(out, '''list { "key": [ "value"\\, "value2" ] };
''')
        
    def test_add_object(self):
        out = runPd(os.path.join(self.basePath, 'add-object.pd'))
        self.assertEquals(out, '''list { "key": { "id": "test"\, "name": "my name" } };
''')
        
    def test_clear(self):
        out = runPd(os.path.join(self.basePath, 'clear.pd'))
        self.assertEquals(out, '''list ;
''')
        
    def test_empty_bang(self):
        out = runPd(os.path.join(self.basePath, 'empty-bang.pd'))
        self.assertEquals(out, '''list ;
''')
        
    def test_add_int(self):
        out = runPd(os.path.join(self.basePath, 'add-int.pd'))
        self.assertEquals(out, '''list { "key": 1 };
''')
    
    def test_write(self):
        outFile = os.path.join(self.basePath, 'write.json')
        runPd(os.path.join(self.basePath, 'write.pd'))
        fileHandle = open(outFile)
        out = fileHandle.read();
        try:
            os.remove(outFile);
        except:
            pass
        self.assertEquals(out, '{ "key": 1 }')

    def test_read(self):
        out = runPd(os.path.join(self.basePath, 'read.pd'))
        self.assertEquals(out, '''list { "key": 1 };
''')
    
    def test_read_large(self):
        out = runPd(os.path.join(self.basePath, 'read-large.pd'))
        self.assertFalse(out == '''
''')
    
if __name__ == '__main__': 
    unittest.main()



#! /usr/bin/python

import unittest
import os.path
from pdStarter import runPd

class UrlparamsTests(unittest.TestCase):
    basePath = 'urlparams'
    def test_add_single_value(self):
        out = runPd(os.path.join(self.basePath, 'add-single-value.pd'))
        self.assertEquals(out, '''list key=value;
''')

    def test_add_two_values(self):
        out = runPd(os.path.join(self.basePath, 'add-two-values.pd'))
        self.assertEquals(out, '''list key=value&key2=value2;
''')

    def test_replace_value(self):
        out = runPd(os.path.join(self.basePath, 'replace-value.pd'))
        self.assertEquals(out, '''list key=other;
''')

    def test_clear(self):
        out = runPd(os.path.join(self.basePath, 'clear.pd'))
        self.assertEquals(out, '''list ;
''')

    def test_escape_value(self):
        out = runPd(os.path.join(self.basePath, 'escape-value.pd'))
        self.assertEquals(out, '''list key=value%20with%20%23spaces%26other%2bst%c3%bcff%20incl.%20%c3%9cml%c3%a4ute;
''')

    def test_escape_key(self):
        out = runPd(os.path.join(self.basePath, 'escape-key.pd'))
        self.assertEquals(out, '''list key%23%24=value;
''')

if __name__ == '__main__': 
    unittest.main()



#! /usr/bin/python

import subprocess

def runPd(pdPatch):
    pdStartup = 'pd'
    pd = subprocess.Popen([pdStartup, '-nosound', '-batch', '-send', 'unittest bang', pdPatch],
            stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    out, err = pd.communicate()
    return out



#! /usr/bin/python

import unittest
import os.path
from pdStarter import runPd

class JsonDecodeTests(unittest.TestCase):
    basePath = 'json-decode'
    def test_simple_object(self):
        out = runPd(os.path.join(self.basePath, 'simple-object.pd'))
        self.assertEquals(out, '''list key value;
bang;
''')

    def test_multiple_members(self):
        out = runPd(os.path.join(self.basePath, 'multiple-members.pd'))
        self.assertEquals(out, '''list key value;
list key2 2;
list key3 0.1;
list bool 0;
bang;
''')

    def test_array(self):
        out = runPd(os.path.join(self.basePath, 'array.pd'))
        self.assertEquals(out, '''list key value;
list id 1;
bang;
list key value 2;
list id 2;
bang;
''')

if __name__ == '__main__': 
    unittest.main()



# Converts the project Wiki to html and adds it as manual.
#
# Adapted from https://gist.github.com/mrexmelle/659abc02ae1295d60647

import os
import subprocess
import sys
from bs4 import BeautifulSoup
from grip import export

wikiDir = '/tmp/PuRestJson.wiki/'
exportDir = 'manual/'

if len(sys.argv) > 1:
    exportDir = sys.argv[1]

if len(sys.argv) > 2:
    wikiDir = sys.argv[2]

print ('Input directory: ', wikiDir)
print ('Output directory: ', exportDir)

for f in os.listdir(wikiDir):
    if f.endswith('.md'):
        print ('Converting: ', f)
        baseFile = os.path.splitext(os.path.basename(f))[0]
        htmlFile = baseFile + '.html'
        export(wikiDir + f, render_inline=False,
               out_filename=exportDir + htmlFile, 
               title=baseFile.replace('-', ' '))
        # edit links to css, images and other pages.
        htmlDoc = open(exportDir + htmlFile)
        soup = BeautifulSoup(htmlDoc, 'lxml')
        for s in soup.findAll('link'):
            s.extract()
        css = soup.new_tag('link')
        css.attrs['rel'] = 'stylesheet'
        css.attrs['href'] = 'style.css'
        soup.head.append(css)
        # internal links.
        for a in soup.findAll('a'):
            if a['href'].startswith('https://github.com/residuum/PuRestJson/wiki/'):
                a['href'] = a['href']\
                        .replace('https://github.com/residuum/PuRestJson/wiki/', '')\
                        .replace('%5B', '')\
                        .replace('%5D', '')\
                        .replace(':', '') + '.html'
        # images with link to itself.
        for img in soup.findAll('img'):
            if img['src'].startswith('https://camo.githubusercontent.com'):
                img['src'] = img['data-canonical-src']\
                        .replace('https://raw.github.com/residuum/PuRestJson/master/manual/','')
                del img['data-canonical-src']
                a = img.parent
                del a['href']
        # write changes back to file
        htmlDoc.close()
        html = soup.prettify('utf-8')
        with open(exportDir + htmlFile, 'wb') as edited:
            edited.write(html)
        # rename all files containing '[' and ']' in names,
        # because Windows does not like those.
        cleaned = htmlFile.replace('[', '').replace(']', '').replace(':', '')
        if htmlFile != cleaned:
            os.rename(exportDir + htmlFile, exportDir + cleaned)

os.rename(exportDir + "Home.html", exportDir + "index.html")
